use crate::utils::mime::mime_enum::Mime;

use super::datatypes::Request;

impl Request<'_> {
    pub fn extract_mime_from_vec(headers: &Vec<String>) -> Option<Mime> {
        let Some(content_type_header) = headers
            .iter()
            .find(|header| header.starts_with("Content-Type: ")) else {
            return None;
        };
        let content_type_string = content_type_header
            .strip_prefix("Content-Type: ")
            .unwrap()
            .to_string();

        let mime;
        match content_type_string.split_once(';') {
            Some(sub) => {
                mime = if let Ok(a) = sub.0.trim().parse() {
                    a
                } else {
                    return None;
                };
            }
            None => {
                mime = if let Ok(a) = content_type_string.trim().parse() {
                    a
                } else {
                    return None;
                };
            }
        }
        Some(mime)
    }
}

#[cfg(test)]
mod test {
    use crate::{handling::request::Request, utils::mime::mime_enum::Mime};

    #[test]
    pub fn test_mime_parse_from_header_vec() {
        let mut right = vec![
            "GET / 23".to_string(),
            "SDF:LKJSD:F".to_string(),
            "Content-Type: text/plain".to_string(),
            "SDF".to_string(),
        ];
        let mut wrong = vec!["SDF:LKJSD:F".to_string(), "SDF".to_string()];

        assert_eq!(None, Request::extract_mime_from_vec(&mut wrong));
        assert_eq!(
            Mime::TextPlain,
            Request::extract_mime_from_vec(&mut right).unwrap()
        );
        assert_eq!(None, Request::extract_mime_from_vec(&mut right));
    }
}
